(* ******************************************************************** *)
(*
 * Project         : HOL-CSP - A Shallow Embedding of CSP in  Isabelle/HOL
 * Version         : 2.0
 *
 * Author          : Burkhart Wolff
 *                   (Based on HOL-CSP 1.0 by Haykal Tej and Burkhart Wolff)
 *
 * This file       : deterministic (external) choice operator definition
 *
 * Copyright (c) 2009 Universit Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ROOT.ML 8591 2009-09-26 09:58:14Z brucker $ *)

header{* Deterministic Choice Operator Definition *}

theory  Det 
imports Process (* Legacy *)
begin

definition
        det      :: "['\<alpha> process,'\<alpha> process] \<Rightarrow> '\<alpha> process"   (infixl "[+]" 18)
where   "P [+] Q \<equiv> Abs_Process(  {(s,X). s = [] \<and> (s,X) \<in> F P \<inter> F Q}
                               \<union> {(s,X). s \<noteq> [] \<and> (s,X) \<in> F P \<union> F Q}
                               \<union> {(s,X). s = [] \<and> s \<in> D P \<union> D Q}
                               \<union> {(s,X). s = [] \<and> tick \<notin> X \<and> [tick] \<in> T P \<union> T Q},
                               D P \<union> D Q)"

notation(xsymbol)
  det (infixl "\<box>" 18)

axioms 
  F_ndet    : "F(P [+] Q) = {(s,X). s = [] \<and> (s,X) \<in> F P \<inter> F Q}
                            \<union> {(s,X). s \<noteq> [] \<and> (s,X) \<in> F P \<union> F Q}
                            \<union> {(s,X). s = [] \<and> s \<in> D P \<union> D Q}
                            \<union> {(s,X). s = [] \<and> tick \<notin> X \<and> [tick] \<in> T P \<union> T Q}"
  D_ndet    : "D(P [+] Q) = D P \<union> D Q"
  T_ndet    : "T(P [+] Q) = T P \<union> T Q"
  ndet_cont : "[| cont f; cont g |] ==> cont (\<lambda>x. f x [+] g x)"


end

