(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ethz-integer-base.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  "ethz-integer-base"
imports 
  FWTesting
begin



(* The subnetworks *)

(* 
 Mapping to the subnetworks

 129.132.240.0/26     \<Longrightarrow>  1
 129.132.176.160/27   \<Longrightarrow>  2
 FW-LINK              \<Longrightarrow>  3
 129.132.254.33/32    \<Longrightarrow>  4
 224.0.0.0/4          \<Longrightarrow>  5
 SIP-Server           \<Longrightarrow>  6
 time.ethz.ch         \<Longrightarrow>  7
 DHCP                 \<Longrightarrow>  8
 129.132.105.4/32     \<Longrightarrow>  9
 129.132.105.10/32    \<Longrightarrow>  10

 ETHZ-Intern          \<Longrightarrow>  11
 ETHZ-Private         \<Longrightarrow>  12
 VPN-VPZ              \<Longrightarrow>  13


*)

constdefs
 FWLink :: "IntegerPort net"
"FWLink \<equiv> {{(a,b). a = 3}}"

 any :: "IntegerPort net"
"any \<equiv> {{(a,b). a > 13}}"

 i26:: "IntegerPort net"
"i26 \<equiv> {{(a,b). a = 1 }}"

 i27:: "IntegerPort net"
"i27 \<equiv> {{(a,b). a = 2 }}"

 i32:: "IntegerPort net"
"i32 \<equiv> {{(a,b). a =  4}}"

 i4:: "IntegerPort net"
"i4 \<equiv> {{(a,b). a = 5 }}"

 sip:: "IntegerPort net"
"sip \<equiv> {{(a,b). a = 6 }}"

 time:: "IntegerPort net"
"time \<equiv> {{(a,b). a = 7}}"

 dhcp:: "IntegerPort net"
"dhcp \<equiv> {{(a,b). a =  8}}"
 
 i4_32:: "IntegerPort net"
"i4_32 \<equiv> {{(a,b). a =  9}}"

 i10_32:: "IntegerPort net"
"i10_32 \<equiv> {{(a,b). a = 10 }}"

 eth_intern:: "IntegerPort net"
"eth_intern \<equiv> {{(a,b). a = 11 }}"
 
 eth_private:: "IntegerPort net"
"eth_private \<equiv> {{(a,b). a = 12 }}"

 vpn_vpz :: "IntegerPort net"
"vpn_vpz \<equiv> {{(a,b). a = 13}}"


constdefs
(* Mandatory: Global *)

 MG1 :: "(IntegerPort,DummyContent) Policy"
"MG1 \<equiv> allow_from_to_port (1::port) i26 any"

 MG2 :: "(IntegerPort,DummyContent) Policy"
"MG2 \<equiv> allow_from_to_ports {(1::port),(2::port),(4::port)} i27 any"

 MG3 :: "(IntegerPort,DummyContent) Policy"
"MG3 \<equiv> allow_from_to_port (1::port) any FWLink"

 MG4 :: "(IntegerPort,DummyContent) Policy"
"MG4 \<equiv> allow_from_to_port (3::port) FWLink FWLink
"
 MG5 :: "(IntegerPort,DummyContent) Policy"
"MG5 \<equiv> allow_from_to_port (5::port) i32 any"

 MG6 :: "(IntegerPort,DummyContent) Policy"
"MG6 \<equiv> allow_from_to_port (6::port) i32 i4 "
 
 MG7 :: "(IntegerPort,DummyContent) Policy"
"MG7 \<equiv> allow_from_to_ports {(6::port),(7::port)} FWLink i4"

 MG8 :: "(IntegerPort,DummyContent) Policy"
"MG8 \<equiv> allow_from_to_ports {(6::port),(7::port)} FWLink i4"

 MG9 :: "(IntegerPort,DummyContent) Policy"
"MG9 \<equiv> allow_from_to_port (5::port) any i32 "

 MG10 :: "(IntegerPort,DummyContent) Policy"
"MG10 \<equiv> allow_from_to_port (5::port) i32 any"

 MG11 :: "(IntegerPort,DummyContent) Policy"
"MG11 \<equiv> allow_from_to_port (1::port) FWLink any"

 MG12 :: "(IntegerPort,DummyContent) Policy"
"MG12 \<equiv> allow_from_to_port (3::port) FWLink FWLink"

 MG13 :: "(IntegerPort,DummyContent) Policy"
"MG13 \<equiv> allow_from_to_ports {(4::port), (2::port)} sip any "

(* Default Global *)

 DG1 :: "(IntegerPort,DummyContent) Policy"
"DG1 \<equiv> allow_from_to_port (8::port) time any "

 DG2:: "(IntegerPort,DummyContent) Policy"
"DG2 \<equiv> allow_from_to_port (1::port) dhcp any "

 DG3:: "(IntegerPort,DummyContent) Policy"
"DG3 \<equiv> deny_from_to_port (7::port) any any "

 


constdefs 
 MandatoryGlobalPolicy :: "(IntegerPort,DummyContent) Policy"
"MandatoryGlobalPolicy \<equiv> MG13 ++ MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8 ++  MG7 ++  MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"   
      
 MandatoryGlobalInsidePolicy :: "(IntegerPort,DummyContent) Policy"
"MandatoryGlobalInsidePolicy \<equiv> MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8"
         
 MandatoryGlobalOutsidePolicy :: "(IntegerPort,DummyContent) Policy"
"MandatoryGlobalOutsidePolicy \<equiv>   MG13 ++ MG7 ++ MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"       

 DefaultGlobalPolicy :: "(IntegerPort,DummyContent) Policy"
"DefaultGlobalPolicy \<equiv> DG3 ++ DG2 ++ DG1"  

 lemmas PolicyLemmas = IntegerPortLemmas   
 src_def dest_def content_def in_subnet_def 
 FWLink_def
 any_def
 i26_def
 i27_def
 i32_def
 i4_def
 sip_def
 time_def
 dhcp_def
 i4_32_def
 i10_32_def
 eth_intern_def
 eth_private_def
 vpn_vpz_def
 MG1_def MG2_def MG3_def MG4_def MG5_def MG6_def MG7_def MG8_def MG9_def MG10_def MG11_def MG12_def MG13_def
 DG1_def DG2_def DG3_def
 MandatoryGlobalPolicy_def MandatoryGlobalOutsidePolicy_def MandatoryGlobalInsidePolicy_def DefaultGlobalPolicy_def

constdefs 
 not_in_same_net :: "(IntegerPort,DummyContent) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv>  ((src x \<sqsubset> i26 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i26 ))) \<and>  
                      ((src x \<sqsubset> i27) \<longrightarrow> ( \<not> (dest x \<sqsubset> i27))) \<and>
                      ((src x \<sqsubset> i32 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i32))) \<and>
                      ((src x \<sqsubset>  i4) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4))) \<and>
                      ((src x \<sqsubset>  sip) \<longrightarrow> ( \<not> (dest x \<sqsubset> sip))) \<and>
                      ((src x \<sqsubset>  time) \<longrightarrow> ( \<not> (dest x \<sqsubset> time))) \<and>
                      ((src x \<sqsubset>  dhcp) \<longrightarrow> ( \<not> (dest x \<sqsubset> dhcp))) \<and>
                      ((src x \<sqsubset>  i4_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4_32))) \<and>
                      ((src x \<sqsubset>  i10_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i10_32))) \<and>
                      ((src x \<sqsubset>  eth_intern) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_intern))) \<and>
                      ((src x \<sqsubset>  vpn_vpz) \<longrightarrow> ( \<not> (dest x \<sqsubset> vpn_vpz))) \<and>
                      ((src x \<sqsubset>  eth_private) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_private)))"
                   
(* Commented out as others import it - would take too long
*)

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas  PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "DG"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalInsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MGI"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalOutsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MGO"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MG"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy ++ MandatoryGLobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas  PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "all"



end
